@@ -7,10 +7,13 @@ import fnmatch
 import os
 import sys
 import textwrap
+import warnings
 
 import coverage
+from coverage import env
 from coverage.backward import StringIO
 from coverage.misc import CoverageException
+from coverage.report import Reporter
 
 from tests.coveragetest import CoverageTest
 
@@ -278,21 +281,20 @@ class ApiTest(CoverageTest):
         self.start_import_stop(cov, "code2")
         self.check_code1_code2(cov)
 
-    if 0:   # expected failure
-        # for https://bitbucket.org/ned/coveragepy/issue/79
-        def test_start_save_stop(self):
-            self.make_code1_code2()
-            cov = coverage.Coverage()
-            cov.start()
-            self.import_local_file("code1")
-            cov.save()
-            self.import_local_file("code2")
-            cov.stop()
+    def test_start_save_stop(self):
+        self.skipTest("Expected failure: https://bitbucket.org/ned/coveragepy/issue/79")
+        self.make_code1_code2()
+        cov = coverage.Coverage()
+        cov.start()
+        self.import_local_file("code1")
+        cov.save()
+        self.import_local_file("code2")
+        cov.stop()
 
-            self.check_code1_code2(cov)
+        self.check_code1_code2(cov)
 
-    def make_corrupt_data_files(self):
-        """Make some good and some bad data files."""
+    def make_good_data_files(self):
+        """Make some good data files."""
         self.make_code1_code2()
         cov = coverage.Coverage(data_suffix=True)
         self.start_import_stop(cov, "code1")
@@ -302,12 +304,15 @@ class ApiTest(CoverageTest):
         self.start_import_stop(cov, "code2")
         cov.save()
 
+    def make_bad_data_file(self):
+        """Make one bad data file."""
         self.make_file(".coverage.foo", """La la la, this isn't coverage data!""")
 
     def test_combining_corrupt_data(self):
         # If you combine a corrupt data file, then you will get a warning,
         # and the file will remain.
-        self.make_corrupt_data_files()
+        self.make_good_data_files()
+        self.make_bad_data_file()
         cov = coverage.Coverage()
         warning_regex = (
             r"Couldn't read data from '.*\.coverage\.foo': "
@@ -322,6 +327,46 @@ class ApiTest(CoverageTest):
         # The bad file still exists.
         self.assert_exists(".coverage.foo")
 
+    def test_combining_twice(self):
+        self.make_good_data_files()
+        cov1 = coverage.Coverage()
+        cov1.combine()
+        cov1.save()
+        self.check_code1_code2(cov1)
+
+        cov2 = coverage.Coverage()
+        with self.assertRaisesRegex(CoverageException, r"No data to combine"):
+            cov2.combine(strict=True)
+
+        cov3 = coverage.Coverage()
+        cov3.combine()
+        # Now the data is empty!
+        _, statements, missing, _ = cov3.analysis("code1.py")
+        self.assertEqual(statements, [1])
+        self.assertEqual(missing, [1])
+        _, statements, missing, _ = cov3.analysis("code2.py")
+        self.assertEqual(statements, [1, 2])
+        self.assertEqual(missing, [1, 2])
+
+
+class NamespaceModuleTest(CoverageTest):
+    """Test PEP-420 namespace modules."""
+
+    def setUp(self):
+        super(NamespaceModuleTest, self).setUp()
+        if env.PYVERSION < (3, 3):
+            self.skipTest("Python before 3.3 doesn't have namespace packages")
+
+    def test_explicit_namespace_module(self):
+        self.make_file("namespace/package/module.py", "VAR = 1\n")
+        self.make_file("main.py", "import namespace\n")
+
+        cov = coverage.Coverage()
+        self.start_import_stop(cov, "main")
+
+        with self.assertRaisesRegex(CoverageException, r"Module .* has no file"):
+            cov.analysis(sys.modules['namespace'])
+
 
 class UsingModulesMixin(object):
     """A mixin for importing modules from test/modules and test/moremodules."""
@@ -407,7 +452,7 @@ class SourceOmitIncludeTest(OmitIncludeTestsMixin, CoverageTest):
         """
         cov = coverage.Coverage(**kwargs)
         cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error,unused-variable
+        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         data = cov.get_data()
         summary = data.line_counts()
@@ -446,7 +491,7 @@ class ReportIncludeOmitTest(OmitIncludeTestsMixin, CoverageTest):
         """Try coverage.report()."""
         cov = coverage.Coverage()
         cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error,unused-variable
+        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         report = StringIO()
         cov.report(file=report, **kwargs)
@@ -465,7 +510,7 @@ class XmlIncludeOmitTest(OmitIncludeTestsMixin, CoverageTest):
         """Try coverage.xml_report()."""
         cov = coverage.Coverage()
         cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error,unused-variable
+        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         cov.xml_report(outfile="-", **kwargs)
         return self.stdout()
@@ -528,7 +573,7 @@ class TestRunnerPluginTest(CoverageTest):
         self.start_import_stop(cov, "no_biggie")
         cov.combine()
         cov.save()
-        cov.report(["no_biggie.py"])
+        cov.report(["no_biggie.py"], show_missing=True)
         self.assertEqual(self.stdout(), textwrap.dedent("""\
             Name           Stmts   Miss  Cover   Missing
             --------------------------------------------
@@ -540,3 +585,20 @@ class TestRunnerPluginTest(CoverageTest):
 
     def test_nose_plugin_with_erase(self):
         self.pretend_to_be_nose_with_cover(erase=True)
+
+
+class ReporterDeprecatedAttributeTest(CoverageTest):
+    """Test that Reporter.file_reporters has been deprecated."""
+
+    run_in_temp_dir = False
+
+    def test_reporter_file_reporters(self):
+        rep = Reporter(None, None)
+
+        with warnings.catch_warnings(record=True) as warns:
+            warnings.simplefilter("always")
+            # Accessing this attribute will raise a DeprecationWarning.
+            rep.file_reporters      # pylint: disable=pointless-statement
+
+        self.assertEqual(len(warns), 1)
+        self.assertTrue(issubclass(warns[0].category, DeprecationWarning))
